<?php

declare(strict_types=1);

namespace Drupal\plain_search_index_filter;

use Twig\Extension\AbstractExtension;
use Twig\Markup;
use Twig\TwigFilter;

/**
 * Twig extension.
 */
final class PlainSearchIndexFilterTwigExtension extends AbstractExtension {

  /**
   * Returns the list of custom filters this extension offers.
   *
   * @return array
   */
  public function getFilters(): array {
    return [
      new TwigFilter('strip_tags_safe', [$this, 'stripTagsSafe']),
    ];
  }

  /**
   * Cleans the given string by safely removing HTML and other undesirable
   * elements. The method handles instances of \Twig\Markup by converting them
   * into a string, strips HTML tags, removes certain inline styles and
   * scripts, handles whitespace effectively, and ensures consistent
   * formatting.
   *
   * @param mixed $string The input data, which can be a string or an instance
   *   of \Twig\Markup.
   *
   * @return mixed The sanitized string if the input was a string, or the
   *   original input otherwise.
   */
  public function stripTagsSafe(mixed $string): mixed {
    // Detect if Twig\Markup and if so convert to string.
    if ($string instanceof Markup) {
      $string = (string) $string;
    }
    if (is_string($string)) {
      // Remove any none breaking space characters, replacing with a space.
      $string = str_replace('&nbsp;', ' ', $string);
      // This line ensures proper spacing before closing HTML tags by adding a space
      // before the closing tag if it is incorrectly formatted (e.g., "< /tag>" to "</tag>").
      $string = preg_replace('/<\s*\/\s*([^>]+)>/', ' </$1>', $string);
      // Regex to remove <script>...</script> and <style>...</style>
      $string = preg_replace('/<script\b[^>]*>(.*?)<\/script>/is', '', $string);
      $string = preg_replace('/<style\b[^>]*>(.*?)<\/style>/is', '', $string);
      // Now we need to strip tags from the string.
      // At this point I'm not allowing for this to be passed in via the filter, just strip all tags.
      $string = strip_tags($string);
      // Remove tabs, swapping for spaces
      $string = str_replace("\t", ' ', $string);
      // Remove multiple \n\n\n replacing with just one.
      $string = preg_replace('/\n\n\n+/', "\n", $string);
      // Remove spare white spaces
      // $string = preg_replace('/\s+/', ' ', $string);
      return $string;
    }
    else {
      // do nothing
      // @todo, I could always render to string,
      // and pass it back as a string at a later date.
      return $string;
    }
  }

}